;If you write a ";" char left most position the assembler will understand that the line
;is information and should not be assembled!
;
;This program below will make RB0 toggle (blink) (pin 21).
;
;
;First I will define some variables to a value.
;This is good because I can then write the lable name instead of the adress number which
;makes it easier to understand and remember.
;
;
;
;
;
STATUS     EQU 3h             ;Define lable STATUS to 3
PORTB      EQU 6h             ;Define lable PORTB  to 6  (Bank 0)
TRISB      EQU 6h             ;Define lable TRISB  to 6  (Bank 1)
;
;TRISB is a register which set the direction of PORTB. It set every pin as input or output. 
;You may wonder how PORTB and TRISB can have same adress 6? THEY DON'T. When bank 0 is set
;you will acces PORTB and when Bank 1 is set you will acces TRISB. Bank selection is made
;in the STATUS register, you will see later in the code.
;
;Here I will define 2 variables in the RAM. They will keep track of the loop counter.
;
COUNT_1    EQU 20h            ;Define lable COUNT_1 to RAM adress 20h
COUNT_2    EQU 21h            ;Define lable COUNT_2 to RAM adress 21h
;
;================== MAIN PROGRAM ==================================
;
;After a reset the processor will go to adress 000h and start execute. What we want first to 
;happen is that the processor jumps to another adress 010h instead, because at adress 004h you
;will have interupt adress and it would not be good if our program got there, so the common
;procedure is to jump a few adresses ahead.
;
STARTADR:  ORG 000h           ;Point to adress 000h which is the start adress
           GOTO START         ;Jump over the fist adresses and to START
;
           ORG 010h           ;Point to adress 010h. 
START:     MOVLW b'00100000'  ;Load W register so bank 1 will be set (see statusreg in datasheets)
           MOVWF STATUS       ;Move W into STATUS and now Bank 1 will be selected
           MOVLW b'11111110'  ;Load W with data direction for TRISB 1=input 0=output
           MOVWF TRISB        ;Move W into TRISB and RB0 will be an output
           MOVLW b'00000000'  ;Load W register so bank 0 will be set (see statusreg in datasheets)
           MOVWF STATUS       ;Move W into STATUS and now Bank 0 will be selected
;
;What we have done so far is to change bank from 0 to 1 and back to 0,
;and made portB RBO (pin21) to output. The rest of portB are inputs.     
;
;Since the PIC execute so many instruction/sec I have to make a long loop between I toggle
;the output to the LED else you will not be able to see the blinking.
;I have solved this by making a nested loop. It means I have made one loop in another loop.
;Count_2 will count down from ffh to 00 and when it reach 00h count_1 will decrese by one.
;The whole loop will end when count_1 reach 00h.
;
;But first I will make the LED light
;
BACK:      MOVLW b'00000001'  ;Load W with data so the output will go high.
           MOVWF PORTB        ;Move W to port B and now RBO will go high.
;Loop
           MOVLW 0ffh         ;Load W with Loop time
           MOVWF COUNT_1      ;Move W to RAM adress

NEXT_1:    MOVLW 0ffh         ;Load W with Loop time
           MOVWF COUNT_2      ;Move W to RAM adress
NEXT_2:    DECFSZ COUNT_2,F   ;Decrease COUNT_2 and execute next instruction if result not zero.
           GOTO NEXT_2        ;
;
           DECFSZ COUNT_1,F   ;Decrease COUNT_1 and execute next instruction if result not zero.
           GOTO NEXT_1        ;
;Loop finished
;
;Now I will turn off the LED
;
           MOVLW b'00000000'  ;Load W with data so the output will go low.
           MOVWF PORTB        ;Move W to port B and now RBO will go low.
;Loop again
           MOVLW 0ffh         ;Load W with Loop time
           MOVWF COUNT_1      ;Move W to RAM adress

NEXT_3:    MOVLW 0ffh         ;Load W with Loop time
           MOVWF COUNT_2      ;Move W to RAM adress
NEXT_4:    DECFSZ COUNT_2,F   ;Decrease COUNT_2 and execute next instruction if result not zero.
           GOTO NEXT_4        ;
;
           DECFSZ COUNT_1,F   ;Decrease COUNT_1 and execute next instruction if result not zero.
           GOTO NEXT_3        ;
;Loop finished
;
;One blink has now occured and the program will jump back to reapeat everything again.
;
           GOTO BACK          ;Make the microcontroller stay in this loop forever.
           END                ;END of program or end of assembling
;
;

